"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("@pnpm/logger");
const server_1 = require("@pnpm/server");
const getPort = require("get-port");
const fs = require("graceful-fs");
const isWindows = require("is-windows");
const mkdirp = require("mkdirp-promise");
const path = require("path");
const onExit = require("signal-exit");
const extendOptions_1 = require("supi/lib/api/extendOptions");
const writeJsonFile = require("write-json-file");
const createStore_1 = require("../createStore");
exports.default = (input, opts) => __awaiter(this, void 0, void 0, function* () {
    logger_1.default.warn('The store server is an experimental feature. Breaking changes may happen in non-major versions.');
    const strictOpts = yield extendOptions_1.default(opts);
    const store = yield createStore_1.default(strictOpts);
    // the store folder will be needed because server will want to create a file there
    // for the IPC connection
    yield mkdirp(store.path);
    const serverOptions = yield getServerOptions(store.path);
    const connectionOptions = {
        remotePrefix: serverOptions.path
            ? `http://unix:${serverOptions.path}:`
            : `http://${serverOptions.hostname}:${serverOptions.port}`,
    };
    const serverJsonPath = path.join(store.path, 'server.json');
    yield writeJsonFile(serverJsonPath, { connectionOptions });
    const server = server_1.createServer(store.ctrl, serverOptions);
    onExit(() => {
        server.close();
        fs.unlinkSync(serverJsonPath);
    });
});
function getServerOptions(fsPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (isWindows()) {
            return {
                hostname: 'localhost',
                port: yield getPort({ port: 5813 }),
            };
        }
        return {
            path: path.normalize(fsPath) + path.sep + 'socket',
        };
    });
}
//# sourceMappingURL=server.js.map